@@ -56,7 +56,7 @@ from randomizer.Enums.Levels import Levels
 from randomizer.Enums.SwitchTypes import SwitchType
 from randomizer.Enums.Switches import Switches
 from randomizer.Lists.Switches import SwitchInfo
-from archipelago.Options import Goal, SwitchSanity, SelectStartingKong
+from archipelago.Options import Goal, SwitchSanity, SelectStartingKong, GalleonWaterLevel
 from archipelago.Goals import GOAL_MAPPING, QUANTITY_GOALS, calculate_quantity
 from archipelago.Logic import logic_item_name_to_id
 
@@ -128,7 +128,6 @@ def get_default_settings() -> dict:
         "filler_items_selected": [ItemRandoFiller.junkitem],
         "free_trade_setting": True,
         "fungi_time": FungiTimeSetting.dusk,
-        "galleon_water": GalleonWaterSetting.raised,
         "generate_spoilerlog": True,
         "hard_bosses_selected": [],
         "hard_mode_selected": [],
@@ -151,6 +150,7 @@ def get_default_settings() -> dict:
             ItemRandoListSelected.wrinkly,
             ItemRandoListSelected.gauntletbanana,
             ItemRandoListSelected.racebanana,
+            ItemRandoListSelected.sniderewards,
             ItemRandoListSelected.arenas,
             ItemRandoListSelected.halfmedal,
             ItemRandoListSelected.enemies,
@@ -188,12 +188,11 @@ def get_default_settings() -> dict:
         "kong_model_tiny": KongModels.default,
         "krool_access": False,
         "krool_in_boss_pool": False,
-        "krool_key_count": 8,
+        "krool_key_count": 0,
         "krool_phase_count": 3,
         "krool_phase_order_rando": True,
         "krool_random": False,
         "less_fragile_boulders": True,
-        "level_randomization": LevelRandomization.level_order_complex,
         "logic_type": LogicType.glitchless,
         "maximize_helm_blocker": True,
         "medal_cb_req": 40,
@@ -316,6 +315,7 @@ def apply_archipelago_settings(settings_dict: dict, options, multiworld) -> None
     settings_dict["krool_in_boss_pool"] = options.krool_in_boss_pool.value
     settings_dict["helm_phase_count"] = options.helm_phase_count.value
     settings_dict["krool_phase_count"] = options.krool_phase_count.value
+    settings_dict["level_randomization"] = LevelRandomization.loadingzone if options.loading_zone_rando.value else LevelRandomization.level_order_complex
 
     # Medal distribution settings
     if options.medal_distribution.value == 0:  # pre_selected
@@ -336,6 +336,12 @@ def apply_archipelago_settings(settings_dict: dict, options, multiworld) -> None
     if options.enable_cutscenes.value:
         settings_dict["more_cutscene_skips"] = ExtraCutsceneSkips.press
     settings_dict["alt_minecart_mayhem"] = options.alternate_minecart_mayhem.value
+    if options.galleon_water_level == GalleonWaterLevel.option_lowered:
+        settings_dict["galleon_water"] = GalleonWaterSetting.lowered
+    elif options.galleon_water_level == GalleonWaterLevel.option_raised:
+        settings_dict["galleon_water"] = GalleonWaterSetting.raised
+    else:
+        settings_dict["galleon_water"] = GalleonWaterSetting.vanilla
 
 
 def apply_blocker_settings(settings_dict: dict, options) -> None:
@@ -382,6 +388,7 @@ def apply_item_randomization_settings(settings_dict: dict, options) -> None:
         ItemRandoListSelected.banana,
         ItemRandoListSelected.racebanana,
         ItemRandoListSelected.gauntletbanana,
+        ItemRandoListSelected.blueprintbanana,
         ItemRandoListSelected.crown,
         ItemRandoListSelected.blueprint,
         ItemRandoListSelected.key,
@@ -435,7 +442,8 @@ def apply_hard_mode_settings(settings_dict: dict, options) -> None:
 def apply_kong_settings(settings_dict: dict, options) -> None:
     """Apply Kong settings."""
     # Key settings
-    settings_dict["krool_key_count"] = options.krool_key_count.value
+    settings_dict["krool_key_count"] = options.pregiven_keys.value
+    settings_dict["win_condition_spawns_ship"] = 1 if options.require_beating_krool.value else 0
 
     # Kong mapping
     kong_mapping = {
@@ -677,6 +685,13 @@ def apply_goal_settings(settings_dict: dict, options, random_obj) -> None:
     """Apply goal and win condition settings."""
     settings_dict["win_condition_item"] = GOAL_MAPPING[options.goal]
 
+    # Krool's Challenge always requires beating K. Rool otherwise wheres the challenge
+    if options.goal == Goal.option_krools_challenge:
+        settings_dict["win_condition_spawns_ship"] = True
+    # The rabbit is too powerful to allow this
+    elif options.goal == Goal.option_kill_the_rabbit:
+        settings_dict["win_condition_spawns_ship"] = False
+
     if options.goal in QUANTITY_GOALS.keys():
         goal_name = QUANTITY_GOALS[options.goal]
         settings_dict["win_condition_count"] = calculate_quantity(goal_name, options.goal_quantity.value, random_obj)
@@ -731,6 +746,66 @@ def apply_minigame_settings(settings_dict: dict, options, multiworld) -> None:
     settings_dict["bonus_barrel_auto_complete"] = options.auto_complete_bonus_barrels.value and options.goal.value != Goal.option_bonuses
     settings_dict["helm_room_bonus_count"] = HelmBonuses(options.helm_room_bonus_count.value)
 
+    # Map crown door and coin door settings
+    crown_door_mapping = {
+        0: HelmDoorItem.vanilla,
+        1: HelmDoorItem.opened,
+        2: HelmDoorItem.medium_random,
+        3: HelmDoorItem.req_gb,
+        4: HelmDoorItem.req_bp,
+        5: HelmDoorItem.req_companycoins,
+        6: HelmDoorItem.req_key,
+        7: HelmDoorItem.req_medal,
+        8: HelmDoorItem.req_crown,
+        9: HelmDoorItem.req_fairy,
+        10: HelmDoorItem.req_rainbowcoin,
+        11: HelmDoorItem.req_bean,
+        12: HelmDoorItem.req_pearl,
+        13: HelmDoorItem.easy_random,
+        14: HelmDoorItem.hard_random,
+    }
+
+    coin_door_mapping = {
+        0: HelmDoorItem.vanilla,
+        1: HelmDoorItem.opened,
+        2: HelmDoorItem.medium_random,
+        3: HelmDoorItem.req_gb,
+        4: HelmDoorItem.req_bp,
+        6: HelmDoorItem.req_key,
+        7: HelmDoorItem.req_medal,
+        8: HelmDoorItem.req_crown,
+        9: HelmDoorItem.req_fairy,
+        10: HelmDoorItem.req_rainbowcoin,
+        11: HelmDoorItem.req_bean,
+        12: HelmDoorItem.req_pearl,
+        13: HelmDoorItem.easy_random,
+        14: HelmDoorItem.hard_random,
+    }
+
+    # Map door item type to the key name in helm_door_item_count dict
+    door_item_to_key = {
+        3: "golden_bananas",  # req_gb
+        4: "blueprints",  # req_bp
+        5: "company_coins",  # req_companycoins
+        6: "keys",  # req_key
+        7: "medals",  # req_medal
+        8: "crowns",  # req_crown
+        9: "fairies",  # req_fairy
+        10: "rainbow_coins",  # req_rainbowcoin
+        11: "bean",  # req_bean
+        12: "pearls",  # req_pearl
+    }
+
+    settings_dict["crown_door_item"] = crown_door_mapping.get(options.crown_door_item.value, HelmDoorItem.opened)
+    # Get count from dict based on selected item, default to 1 if not found
+    crown_item_key = door_item_to_key.get(options.crown_door_item.value)
+    settings_dict["crown_door_item_count"] = options.helm_door_item_count.value.get(crown_item_key, 1) if crown_item_key else 1
+
+    settings_dict["coin_door_item"] = coin_door_mapping.get(options.coin_door_item.value, HelmDoorItem.opened)
+    # Get count from dict based on selected item, default to 1 if not found
+    coin_item_key = door_item_to_key.get(options.coin_door_item.value)
+    settings_dict["coin_door_item_count"] = options.helm_door_item_count.value.get(coin_item_key, 1) if coin_item_key else 1
+
     if hasattr(multiworld, "generation_is_fake"):
         if hasattr(multiworld, "re_gen_passthrough"):
             if "Donkey Kong 64" in multiworld.re_gen_passthrough:
@@ -749,10 +824,8 @@ def handle_fake_generation_settings(settings: Settings, multiworld) -> None:
 
                 # Switch logic lifted out of level shuffle due to static levels for UT
                 if settings.alter_switch_allocation:
-                    allocation = [1, 1, 1, 1, 2, 2, 3, 3]
                     for x in range(8):
-                        level = settings.level_order[x + 1]
-                        settings.switch_allocation[level] = allocation[x]
+                        settings.switch_allocation[x] = passthrough["SlamLevels"][x]
 
                 settings.starting_kong_list = passthrough["StartingKongs"]
                 settings.starting_kong = settings.starting_kong_list[0]  # fake a starting kong so that we don't force a different kong
