@@ -72,8 +72,8 @@ ShufflableExits = {
     Transitions.IslesAztecLobbyToMain: ShufflableExit("DK Isles Aztec Lobby to Main", Regions.AngryAztecLobby, TransitionBack(Regions.IslesMainUpper, "From Aztec Lobby", "DK Isles Main from Aztec Lobby", Transitions.IslesMainToAztecLobby), ExitCategories.AztecLobby),
     Transitions.IslesMainToFactoryLobby: ShufflableExit("DK Isles Main to Factory Lobby", Regions.KremIsleBeyondLift, TransitionBack(Regions.FranticFactoryLobby, "From DK Isles", "Factory Lobby from DK Isles Main", Transitions.IslesFactoryLobbyToMain), ExitCategories.IslesExterior),
     Transitions.IslesFactoryLobbyToMain: ShufflableExit("DK Isles Factory Lobby to Main", Regions.FranticFactoryLobby, TransitionBack(Regions.KremIsleBeyondLift, "From Factory Lobby", "DK Isles Main from Factory Lobby", Transitions.IslesMainToFactoryLobby), ExitCategories.FactoryLobby),
-    Transitions.IslesMainToGalleonLobby: ShufflableExit("DK Isles Main to Galleon Lobby", Regions.IslesMain, TransitionBack(Regions.GloomyGalleonLobbyEntrance, "From DK Isles", "Galleon Lobby from DK Isles Main", Transitions.IslesGalleonLobbyToMain), ExitCategories.IslesExterior),
-    Transitions.IslesGalleonLobbyToMain: ShufflableExit("DK Isles Galleon Lobby to Main", Regions.GloomyGalleonLobbyEntrance, TransitionBack(Regions.IslesMain, "From Galleon Lobby", "DK Isles Main from Galleon Lobby", Transitions.IslesMainToGalleonLobby), ExitCategories.GalleonLobby),
+    Transitions.IslesMainToGalleonLobby: ShufflableExit("DK Isles Main to Galleon Lobby", Regions.KremIsle, TransitionBack(Regions.GloomyGalleonLobbyEntrance, "From DK Isles", "Galleon Lobby from DK Isles Main", Transitions.IslesGalleonLobbyToMain), ExitCategories.IslesExterior),
+    Transitions.IslesGalleonLobbyToMain: ShufflableExit("DK Isles Galleon Lobby to Main", Regions.GloomyGalleonLobbyEntrance, TransitionBack(Regions.KremIsle, "From Galleon Lobby", "DK Isles Main from Galleon Lobby", Transitions.IslesMainToGalleonLobby), ExitCategories.GalleonLobby),
     Transitions.IslesMainToForestLobby: ShufflableExit("DK Isles Main to Fungi Lobby", Regions.CabinIsle, TransitionBack(Regions.FungiForestLobby, "From DK Isles", "Fungi Lobby from DK Isles Main", Transitions.IslesForestLobbyToMain), ExitCategories.IslesExterior),
     Transitions.IslesForestLobbyToMain: ShufflableExit("DK Isles Fungi Lobby to Main", Regions.FungiForestLobby, TransitionBack(Regions.CabinIsle, "From Fungi Lobby", "DK Isles Main from Fungi Lobby", Transitions.IslesMainToForestLobby), ExitCategories.ForestLobby),
     Transitions.IslesMainToCavesLobby: ShufflableExit("DK Isles Main to Caves Lobby", Regions.IslesEar, TransitionBack(Regions.CrystalCavesLobby, "From DK Isles", "Caves Lobby from DK Isles Main", Transitions.IslesCavesLobbyToMain), ExitCategories.IslesExterior),
@@ -256,7 +256,7 @@ ShufflableExits = {
     Transitions.CastleMainToUpper: ShufflableExit("Creepy Castle Main to Upper Cave (Back door)", Regions.CreepyCastleMain, TransitionBack(Regions.UpperCave, "From Castle (Rear)", "Creepy Castle Upper Cave from Main (Back door)", Transitions.CastleUpperToMain), ExitCategories.CastleExterior),
     Transitions.CastleUpperToMain: ShufflableExit("Creepy Castle Upper Cave to Main (Back door)", Regions.UpperCave, TransitionBack(Regions.CreepyCastleMain, "From Tunnel (Rear)", "Creepy Castle Main from Upper Cave (Back door)", Transitions.CastleMainToUpper), ExitCategories.CastleUpper),
     Transitions.CastleWaterfallToUpper: ShufflableExit("Creepy Castle Main to Upper Cave (Moat door)", Regions.CastleWaterfall, TransitionBack(Regions.UpperCave, "From Castle (Front)", "Creepy Castle Upper Cave from Main (Moat door)", Transitions.CastleUpperToWaterfall), ExitCategories.CastleExterior),
-    Transitions.CastleUpperToWaterfall: ShufflableExit("Creepy Castle Upper Cave to Main (Moat door)", Regions.UpperCave, TransitionBack(Regions.CreepyCastleMain, "From Tunnel (Front)", "Creepy Castle Main from Upper Cave (Moat door)", Transitions.CastleWaterfallToUpper), ExitCategories.CastleUpper),
+    Transitions.CastleUpperToWaterfall: ShufflableExit("Creepy Castle Upper Cave to Main (Moat door)", Regions.UpperCave, TransitionBack(Regions.CastleWaterfall, "From Tunnel (Front)", "Creepy Castle Main from Upper Cave (Moat door)", Transitions.CastleWaterfallToUpper), ExitCategories.CastleUpper),
     Transitions.CastleBallroomToMuseum: ShufflableExit("Creepy Castle Ballroom to Museum", Regions.Ballroom, TransitionBack(Regions.MuseumBehindGlass, "From Ballroom (Monkeyport)", "Creepy Castle Museum (Monkeyport)", Transitions.CastleMuseumToBallroom), ExitCategories.CastleBallroom, entryKongs={Kongs.tiny}, regionKongs={Kongs.tiny}),
     Transitions.CastleMuseumToBallroom: ShufflableExit("Creepy Castle Museum to Ballroom", Regions.MuseumBehindGlass, TransitionBack(Regions.Ballroom, "From Museum (Monkeyport)", "Creepy Castle Ballroom (Monkeyport)", Transitions.CastleBallroomToMuseum), ExitCategories.CastleCarRaceExterior, entryKongs={Kongs.tiny}, regionKongs={Kongs.tiny}),
     Transitions.CastleMuseumToCarRace: ShufflableExit("Creepy Castle Museum to Car Race", Regions.MuseumBehindGlass, TransitionBack(Regions.CastleTinyRace, "From Museum", "Creepy Castle Car Race", Transitions.CastleRaceToMuseum), ExitCategories.CastleCarRaceExterior, entryKongs={Kongs.tiny}, regionKongs={Kongs.tiny}),
