@@ -24,10 +24,31 @@ def writeTransition(settings: Settings, ROM_COPY: ROM) -> None:
     if len(file_data) == 0:
         return
     selected_transition = settings.random.choice(file_data)
-    im_f = Image.open(BytesIO(bytes(selected_transition[0])))
+    im_f = Image.open(BytesIO(bytes(selected_transition[0]))).convert("RGBA")
+    # Check if image has any low-alpha pixels. If it does, assume shrink mode
+    low_alpha_found = False
+    px = im_f.load()
     w, h = im_f.size
-    if w != 64 or h != 64:
-        return
+    low_alpha_conversion = Image.new(mode="RGBA", size=(w, h))
+    px_new = low_alpha_conversion.load()
+    for y in range(h):
+        for x in range(w):
+            r, g, b, a = px[x, y]
+            if a < 128:
+                low_alpha_found = True
+            intensity = 255 - a
+            px_new[x, y] = (intensity, intensity, intensity, 255)
+    if low_alpha_found:
+        im_f = low_alpha_conversion.transpose(Image.FLIP_LEFT_RIGHT).resize((64, 64))
+        px = im_f.load()
+        for x in range(64):
+            px[0, x] = (255, 255, 255, 255)
+            px[63, x] = (255, 255, 255, 255)
+            px[x, 0] = (255, 255, 255, 255)
+            px[x, 63] = (255, 255, 255, 255)
+    else:
+        if w != 64 or h != 64:
+            return
     settings.custom_transition = selected_transition[1].split("/")[-1]  # File Name
     writeColorImageToROM(im_f, 14, 95, 64, 64, False, TextureFormat.IA4, ROM_COPY)
 
@@ -583,7 +604,7 @@ def writeCustomArcadeSprites(settings: Settings, ROM_COPY: ROM) -> None:
 
 
 def writeCustomReels(settings: Settings, ROM_COPY: ROM) -> None:
-    """Write custom painting files to ROM."""
+    """Write custom reel files to ROM."""
     if js.cosmetics is None:
         return
     if js.cosmetics.reel_sprites is None:
@@ -622,3 +643,130 @@ def writeCustomReels(settings: Settings, ROM_COPY: ROM) -> None:
         for chunk_index, chunk in enumerate(chunks):
             img_index = reel.texture_order[chunk_index]
             writeColorImageToROM(chunk, 25, img_index, chunk_w, chunk_h, True, TextureFormat.RGBA5551, ROM_COPY)
+
+
+kong_colors = [
+    (0xFF, 0xD7, 0x00),
+    (0xFF, 0x00, 0x00),
+    (0x16, 0x99, 0xFF),
+    (0xB0, 0x45, 0xFF),
+    (0x41, 0xFF, 0x25),
+    (0xFF, 0xFF, 0xFF),
+]
+
+
+def writeCustomItemSprites(settings: Settings, ROM_COPY: ROM) -> None:
+    """Write custom coin gif files to ROM."""
+    if js.cosmetics is None:
+        return
+    if js.cosmetics.item_sprites is None:
+        return
+    if js.cosmetic_names.item_sprites is None:
+        return
+    file_data = list(zip(js.cosmetics.item_sprites, js.cosmetic_names.item_sprites))
+    if len(file_data) == 0:
+        return
+    list_pool = file_data.copy()
+    groups = {}
+    group_files = {}
+    for item in list_pool:
+        name = item[1].split("/")[-1].split(".")[0]
+        numbers = [str(x) for x in range(10)]
+        item_name = name
+        item_number = ""
+        error = False
+        while True:
+            if len(item_name) == 0:
+                # Reached the start of the item name without detecting a non-number
+                error = True
+                break
+            if item_name[-1] in numbers:
+                item_number = item_name[-1] + item_number
+                item_name = item_name[:-1]
+            else:
+                break
+        if item_number == "" or error:
+            # Has no number at the end, or is all numbers
+            continue
+        if item_name not in groups:
+            groups[item_name] = []
+            group_files[item_name] = {}
+        groups[item_name].append(int(item_number))
+        group_files[item_name][int(item_number)] = item[0]
+    valid_item_names = []
+    for name, numbers in groups.items():
+        for x in range(len(numbers)):
+            if x not in numbers:
+                groups[name] = list(range(x))
+                break
+        if len(groups[name]) > 0:
+            valid_item_names.append(name)
+    if len(valid_item_names) == 0:
+        return
+    chosen_item = settings.random.choice(valid_item_names)
+    frames_per_frame = len(groups[chosen_item]) / 8  # I promise this variable name makes sense
+    coin_data = [
+        # DK
+        {"table": 7, "image": 0x00E0, "frame": 0, "color": 0},
+        {"table": 7, "image": 0x00E1, "frame": 1, "color": 0},
+        {"table": 7, "image": 0x00E2, "frame": 2, "color": 0},
+        {"table": 7, "image": 0x00E3, "frame": 3, "color": 0},
+        {"table": 7, "image": 0x00E4, "frame": 4, "color": 0},
+        {"table": 7, "image": 0x00E5, "frame": 5, "color": 0},
+        {"table": 7, "image": 0x00E6, "frame": 6, "color": 0},
+        {"table": 7, "image": 0x00E7, "frame": 7, "color": 0},
+        # Diddy
+        {"table": 7, "image": 0x0100, "frame": 0, "color": 1},
+        {"table": 7, "image": 0x0101, "frame": 1, "color": 1},
+        {"table": 7, "image": 0x0102, "frame": 2, "color": 1},
+        {"table": 7, "image": 0x0103, "frame": 3, "color": 1},
+        {"table": 7, "image": 0x0104, "frame": 4, "color": 1},
+        {"table": 7, "image": 0x0105, "frame": 5, "color": 1},
+        {"table": 7, "image": 0x0106, "frame": 6, "color": 1},
+        {"table": 7, "image": 0x0107, "frame": 7, "color": 1},
+        # Lanky
+        {"table": 7, "image": 0x00F8, "frame": 0, "color": 2},
+        {"table": 7, "image": 0x00F9, "frame": 1, "color": 2},
+        {"table": 7, "image": 0x00FA, "frame": 2, "color": 2},
+        {"table": 7, "image": 0x00FB, "frame": 3, "color": 2},
+        {"table": 7, "image": 0x00FC, "frame": 4, "color": 2},
+        {"table": 7, "image": 0x00FD, "frame": 5, "color": 2},
+        {"table": 7, "image": 0x00FE, "frame": 6, "color": 2},
+        {"table": 7, "image": 0x00FF, "frame": 7, "color": 2},
+        # Tiny
+        {"table": 7, "image": 0x00D8, "frame": 0, "color": 3},
+        {"table": 7, "image": 0x00D9, "frame": 1, "color": 3},
+        {"table": 7, "image": 0x00DA, "frame": 2, "color": 3},
+        {"table": 7, "image": 0x00DB, "frame": 3, "color": 3},
+        {"table": 7, "image": 0x00DC, "frame": 4, "color": 3},
+        {"table": 7, "image": 0x00DD, "frame": 5, "color": 3},
+        {"table": 7, "image": 0x00DE, "frame": 6, "color": 3},
+        {"table": 7, "image": 0x00DF, "frame": 7, "color": 3},
+        # Chunky
+        {"table": 7, "image": 0x0108, "frame": 0, "color": 4},
+        {"table": 7, "image": 0x0109, "frame": 1, "color": 4},
+        {"table": 7, "image": 0x010A, "frame": 2, "color": 4},
+        {"table": 7, "image": 0x010B, "frame": 3, "color": 4},
+        {"table": 7, "image": 0x010C, "frame": 4, "color": 4},
+        {"table": 7, "image": 0x010D, "frame": 5, "color": 4},
+        {"table": 7, "image": 0x010E, "frame": 6, "color": 4},
+        {"table": 7, "image": 0x010F, "frame": 7, "color": 4},
+    ]
+
+    for x in range(8):
+        selected_frame = int(frames_per_frame * x)
+        selected_frame_data = group_files[chosen_item][selected_frame]
+        im_f = Image.open(BytesIO(bytes(selected_frame_data))).convert("RGBA")
+        im_f = getImageShrink(im_f, 48, 42)
+        im_f = im_f.transpose(Image.FLIP_TOP_BOTTOM).convert("RGBA")
+        for img_data in coin_data:
+            if img_data["frame"] != x:
+                continue
+            r, g, b, a = im_f.split()
+            gray = Image.merge("RGB", (r, g, b)).convert("L")
+            target_color = kong_colors[img_data["color"]]
+            tinted_r = Image.eval(gray, lambda v: v * target_color[0] // 255)
+            tinted_g = Image.eval(gray, lambda v: v * target_color[1] // 255)
+            tinted_b = Image.eval(gray, lambda v: v * target_color[2] // 255)
+            tinted = Image.merge("RGBA", (tinted_r, tinted_g, tinted_b, a))
+            writeColorImageToROM(tinted, img_data["table"], img_data["image"], 48, 42, False, TextureFormat.RGBA5551, ROM_COPY)
